package hep.aida.web.taglib;

import hep.aida.IPlotter;

/**
 * A top level tag which generates an image containing one or more plots.
 * 
 * @author The AIDA Team @ SLAC
 *
 */
public interface PlotterTag extends StyleProvider {

    /**
     * Set the name for the generated plot. This is an optional attribute. If it
     * is not present then a unique name is automatically generated.
     */
    public void setName(String name);

    /**
     * Set the width of the generated image, in pixels.
     */
    public void setWidth(int width);

    /**
     * Set the height of the generated image, in pixels.
     */
    public void setHeight(int height);

    /**
     * Set the number of horizontal cells in the region grid.
     */
    public void setNx(int nx);

    /**
     * Set the number of vertical cells in the region grid.
     */
    public void setNy(int ny);

    /**
     * Set the default format to use for the generated image. Possible values
     * are png, gif, jpg, pdf, eps, svg, and swf.
     */
    public void setFormat(String format);

    /**
     * Set whether to display links below the generated image to allow
     * downloading in other formats.
     */
    public void setAllowDownload(boolean allowDownload);

    /**
     * Set whether to create an image map on top of the plot; each region has a
     * link.
     */
    public void setCreateImageMap(boolean createImageMap);

    /**
     * Set the plotter to translate into an image. This can be an instance of
     * {@link IPlotter}, or the name of a variable in a JSP scope holding an
     * {@link IPlotter}. This is an optional attribute. The {@link IPlotter}
     * generated by any nested <code>&lt;region&gt;</code> and
     * <code>&lt;plot&gt;</code> tags will take precedence.
     */
    public void setVar(Object var);
}